/* Keeps null-terminated string for C api compatibility and tracks the actual
   string length and provides functions for inserting and deleting at
   arbitrary positions, and grows the storage automatically */

#ifndef EDITSTRING_H
#define EDITSTRING_H

#include <stdlib.h>

struct EditString
{
    char *string;
    int storage_size, length;
    int autoresizes;
    int cursor_pos;
};

struct EditString *editstring_new (int max_string_length, int autoresizes);
struct EditString *editstring_copy (struct EditString *old);
struct EditString *editstring_new_from_cstr (char *cstr, int autoresizes);
void editstring_free (struct EditString *ms);

char *editstring_relative_substr (struct EditString *ms, int back, int len);
int editstring_insert_char (struct EditString *ms, char ch);
int editstring_delete_prev (struct EditString *ms);
int editstring_delete_at (struct EditString *ms);
int editstring_set_cstr (struct EditString *ms, char *cstr);
int editstring_set_cstr_with_length (struct EditString *ms, char *cstr, int len);
int editstring_grow (struct EditString *ms, int len);

int editstring_advance_cursor_pos (struct EditString *ms, int change);
int editstring_set_cursor_pos (struct EditString *ms, int pos);

#define AUTORESIZE 1
#define NOAUTORESIZE 0

#endif /* EDITSTRING_H */
