#ifndef STATE_H
#define STATE_H

#include "editstr.h"

#define REQUEST_DUPLICATE_CARD 1
#define REQUEST_GRACEFUL_EXIT 2
#define REQUEST_INSERT 3
#define REQUEST_PREVIOUS_CARD 4
#define REQUEST_NEXT_CARD 5
#define REQUEST_MOVE_CURSOR 6
#define REQUEST_NEW_CARD 7
#define REQUEST_FORCE_REFRESH 8
#define REQUEST_SAVE 9
#define REQUEST_DELETE_PREV 10
#define REQUEST_DELETE_AT 11
#define REQUEST_INFORM_HEIGHT 12
#define REQUEST_INFORM_WIDTH 13
#define REQUEST_DELETE_CARD 14
#define REQUEST_RESIZE 15
#define REQUEST_DIALOG_YES 16
#define REQUEST_DIALOG_NO 17
#define REQUEST_DIALOG_ACCEPT 18
#define REQUEST_DIALOG_ABORT 19
#define REQUEST_EXIT_NOW 20
#define REQUEST_PDF 21

#define DOWN 1
#define LEFT 2
#define RIGHT 3
#define UP 4

#define BKGD_ATTR 0
#define CARD_ATTR 0
#define TEXT_ATTR 0
#define INDEX_ATTR 0
#define HILITE_ATTR A_REVERSE

#define PDF_ALL 1
#define PDF_ONE 2
#define PDF_LETTER 4
#define PDF_A4 8

struct state
{
    struct card *cur_card;
    struct EditString *filename;
    int screen_cols, screen_rows;
    struct card *first_visible, *last_visible;
    enum { MENU, TEMP_STATUS, DIALOG } status;
    enum { TITLE_MODE, BODY_MODE, DIALOG_YN, DIALOG_STRING } input_mode, old_mode;
    char *status_text;
    char *dialog_prompt;
    int dialog_cursor_offset;
    void (*dialog_yes)(struct state *s);
    void (*dialog_no)(struct state *s);
    void (*dialog_abort)(struct state *s);
    void (*dialog_accept)(struct state *s);
    struct EditString *dialog_input;
    int pdf_state;
};

struct state *state_new (void);
void state_determine_visible_titles (struct state *s);
void state_duplicate_card (struct state *s);
void state_read_cardfile (struct state *s, char *filename);
void state_insert (struct state *s, char ch);
void state_previous_card (struct state *s, int refresh_ok);
void state_next_card (struct state *s, int refresh_ok);
void state_jump_card (struct state *s, struct card *c, int refresh_ok);
void state_move_cursor (struct state *s, int dir);
void state_new_card (struct state *s);
void state_delete_card (struct state *s);
void state_force_refresh (struct state *s);
void state_save (struct state *s);
void state_delete_prev (struct state *s);
void state_delete_at (struct state *s);
void state_free (struct state *s);
void state_inform_width (struct state *s, int w);
void state_inform_height (struct state *s, int h);
void state_dialog_yes (struct state *s);
void state_dialog_no (struct state *s);
void state_dialog_abort (struct state *s);
void state_dialog_accept (struct state *s);
void state_graceful_exit (struct state *s);
void state_status_update (struct state *s);
void state_pdf (struct state *s);

/* For state_step_card */
#define NO_REFRESH 0
#define REFRESH_OK 1

#endif /* STATE_H */
