#include "card.h"

#include <assert.h>
#include <stdlib.h>

struct card *alloc_blank_card (void)
{
    struct card *card;

    if ((card = calloc (1, sizeof (struct card))) == NULL)
        return NULL;

    card->title = editstring_new (TITLE_WIDTH, NOAUTORESIZE);
    card->body = multiedit_new (BODY_WIDTH, BODY_HEIGHT, MAX_BODY_BYTES);

    return card;
}

int duplicate_card (struct card *card)
{
    struct card *newcard;

    if ((newcard = malloc (sizeof (struct card))) == NULL)
        return 0;

    *newcard = *card;
    newcard->title = editstring_copy (card->title);
    newcard->body = multiedit_copy (card->body);
    newcard->dirty = 1;

    if (newcard->prev)
        newcard->prev->next = newcard;
    newcard->next = card;
    card->prev = newcard;

    return 1;
}

struct card *delete_card (struct card *card)
{
    struct card *ret;

    if (! card)
        return NULL;

    /* Set the dirty flag on the neighboring cards so that the user will
       be prompted to save on exit */
    if (card->prev)
    {
        card->prev->next = card->next;
        card->prev->dirty = 1;
    }
    if (card->next)
    {
        card->next->prev = card->prev;
        card->next->dirty = 1;
    }

    editstring_free (card->title);
    multiedit_free (card->body);

    ret = card->next ? card->next : card->prev;
    free (card);

    return ret;
}

/* Pass in '\n' to insert newline, returns success */
int insert_body_char (struct card *card, char ch)
{
    if (multiedit_insert_char (card->body, ch))
    {
        card->dirty = 1;
        return 1;
    }
    else
        return 0;
}

int insert_title_char (struct card *card, char ch)
{
    assert (ch != '\n');

    if (editstring_insert_char (card->title, ch))
    {
        card->dirty = 1;
        return 1;
    }
    else
        return 0;
}

int delete_body_at (struct card *c)
{
    if (multiedit_delete_at (c->body))
    {
        c->dirty = 1;
        return 1;
    }
    else
        return 0;
}

int delete_title_at (struct card *c)
{
    if (editstring_delete_at (c->title))
    {
        c->dirty = 1;
        return 1;
    }
    else
        return 0;
}

int delete_body_prev (struct card *c)
{
    if (multiedit_delete_prev (c->body))
    {
        c->dirty = 1;
        return 1;
    }
    else
        return 0;
}

int delete_title_prev (struct card *c)
{
    if (editstring_delete_prev (c->title))
    {
        c->dirty = 1;
        return 1;
    }
    else
        return 0;
}

int get_row (struct card *card, char *buffer, int *row_len, int row)
{
    return multiedit_get_row (card->body, buffer, row_len, row);
}

void reset_cursor_pos (struct card *c, int which) {
    switch(which) {
        case SET_BOTH_TO_ZERO:
            editstring_set_cursor_pos (c->title, 0);
            multiedit_set_cursor_row (c->body, 0);
            multiedit_set_cursor_col (c->body, 0);
            break;
        case SET_TITLE_TO_BODY:
            editstring_set_cursor_pos (c->title, c->body->true_cursor_col);
            break;
        case SET_BODY_TO_TITLE:
            multiedit_set_cursor_col (c->body, c->title->cursor_pos);
            break;
    }
}
