#include "curses.h"

#include <assert.h>
#include <bios.h>
#include <stdlib.h>
#include <string.h>

#pragma check_pointer (off)

char _far *screen_ptr = (char _far *) 0xb8000000L;
char _far *kb_mode_ptr = (char _far *) 0x00400096L;

static char *screen_save;
static int col_save, row_save;

static char cur_attr;

/*int my_getch (void)
{
    unsigned u;
    int kread = _KEYBRD_READ;

    if (*kb_mode_ptr & 16)
	/* Enhanced keyboard present * /
	kread = _NKEYBRD_READ;

    u = _bios_keybrd (kread);
    if ((u & 0xff) == 0) return (int) -(u >> 8);

    return (int) (u & 0xff);
}

int my_kbhit (void)
{
    unsigned u;
    int kready = _KEYBRD_READY;

    if (*kb_mode_ptr & 16)
	/* Enhanced keyboard present * /
	kready = _NKEYBRD_READY;

    u = _bios_keybrd (kready);
    return !! u;
}*/

void move (int row, int col)
{
    unsigned char cc, cr;
    
    cc = (unsigned char) col;
    cr = (unsigned char) row;
    
    _asm
    {
        mov ah, 02h
        mov bh, 00h
        mov dh, cr
        mov dl, cc
        int 10h
    }
}

int get_video_cursor_column (void)
{
    unsigned char cc;
    
    _asm
    {
        mov ah, 03h
        mov bh, 00h
        int 10h
        
        mov cc, dl
    }
    
    return (int) cc;
}

int get_video_cursor_row (void)
{
    unsigned char cr;
    
    _asm
    {
        mov ah, 03h
        mov bh, 00h
        int 10h
        
        mov cr, dh
    }
    
    return (int) cr;
}

int get_video_rows (void)
{
    /* Requires EGA, MCGA or VGA */
    unsigned char _far *ptr = (unsigned char _far *) 0x00400084L;
    return (int)(*ptr) + 1;
}

int get_video_columns (void)
{
    unsigned char cc;
    
    _asm
    {
        mov ah, 0fh
        int 10h
        
        mov cc, ah
    }
    
    return (int) cc;
}

void initscr (void)
{
    int rows, columns;
    size_t num_bytes, i;

    rows = get_video_rows ();
    columns = get_video_columns ();
    num_bytes = (size_t) (rows * columns * 2);
    
    assert (screen_save == NULL);
    
    screen_save = malloc (num_bytes);
    if (!screen_save) exit (1);
    
    for (i = 0; i < num_bytes; i++)
    {
        screen_save[i] = screen_ptr[i];
        screen_ptr[i] = 0;
    }

    col_save = get_video_cursor_column ();
    row_save = get_video_cursor_row ();
}

void endwin (void)
{
    int rows, columns;
    size_t num_bytes, i;

    rows = get_video_rows ();
    columns = get_video_columns ();
    num_bytes = (size_t) (rows * columns * 2);
    
    for (i = 0; i < num_bytes; i++)
        screen_ptr[i] = screen_save[i];

    free (screen_save);
    screen_save = NULL;
    
    move (row_save, col_save);
}

void attrset (int attr)
{
    cur_attr = attr == 0 ? 0x07 : 0x70;
}

void put_at (int x, int y, chtype ch)
{
    int mx, my;
    
    mx = get_video_columns ();
    my = get_video_rows ();

    if (x < 0 || x >= mx || y < 0 || y >= my)
        return;

    screen_ptr[(y * mx + x) * 2] = (char) ch;
    screen_ptr[(y * mx + x) * 2 + 1] = cur_attr;
}

void clear (void)
{
}

void erase (void)
{
}

void refresh (void)
{
}
