/* Variant of EditString for fixed-height, fixed-width with embedded newlines */
/* The newlines are represented by CRLF */

#ifndef MULTIEDIT_H
#define MULTIEDIT_H

#include <stddef.h>

#define SPACES_PER_TAB 8

struct MultiEdit
{
    char *text;
    int text_length, max_text_bytes;

    int control_width, control_height;
    int true_cursor_col, true_cursor_row;
};

struct MultiEdit *multiedit_new (int control_width, int control_height, int max_text_bytes);
struct MultiEdit *multiedit_copy (struct MultiEdit *old);
void multiedit_free (struct MultiEdit *m);

int multiedit_get_row_raw (struct MultiEdit *m, char *buffer, int *row_len, int row);
int multiedit_get_row (struct MultiEdit *m, char *buffer, int *row_len, int row);

int multiedit_insert_char (struct MultiEdit *m, char ch);
int multiedit_delete_prev (struct MultiEdit *m);
int multiedit_delete_at (struct MultiEdit *m);

void multiedit_cursor_left (struct MultiEdit *m);
void multiedit_cursor_right (struct MultiEdit *m);
void multiedit_cursor_up (struct MultiEdit *m);
void multiedit_cursor_down (struct MultiEdit *m);

int multiedit_apparent_cursor_col (struct MultiEdit *m);
int multiedit_apparent_cursor_row (struct MultiEdit *m);

int multiedit_set_cursor_row (struct MultiEdit *m, int row);
int multiedit_set_cursor_col (struct MultiEdit *m, int col);

#endif /* MULTIEDIT_H */
