#include "requests.h"

#include <stdlib.h>

static int post_request_vp_i (int req, void *vp, int i);

struct request
{
    int req;
    void *vp;
    int i;

    struct request *next;
};

static struct request *head, *tail, signal_request;

/* Signal request is a special static used to allow a signal handler
   to post a request since malloc is not signal safe */
/* This has a race condition, not a problem in practice */
int post_signal_request (int req)
{
    /* In use already */
    if (signal_request.next || tail == &signal_request) return 0;

    signal_request.req = req;
    signal_request.vp = 0;
    signal_request.i = 0;
    signal_request.next = NULL;

    if (tail)
    {
        tail->next = &signal_request;
        tail = &signal_request;
    }
    else
    {
        head = &signal_request;
        tail = &signal_request;
    }

    return 1;
}

int post_request (int req)
{
    return post_request_vp_i (req, NULL, 0);
}

int post_request_vp (int req, void *vp)
{
    return post_request_vp_i (req, vp, 0);
}

int post_request_i (int req, int i)
{
    return post_request_vp_i (req, NULL, i);
}

static int post_request_vp_i (int req, void *vp, int i)
{
    struct request *request;

    if ((request = malloc (sizeof (struct request))) == NULL)
        return 0;

    request->req = req;
    request->vp = vp;
    request->i = i;
    request->next = NULL;

    if (tail)
    {
        tail->next = request;
        tail = request;
    }
    else
    {
        head = request;
        tail = request;
    }

    return 1;
}

int any_requests (void)
{
    return ! ! head;
}

int pending_requests_of_type (int req)
{
    struct request *p;
    int c;

    for (p = head, c = 0; p; p = p->next)
        if (p->req == req)
            c++;

    return c;
}

int pop_request (int *request, int *i, void **vp)
{
    struct request *old_head;

    if (!head) return 0;

    *request = head->req;
    *i = head->i;
    *vp = head->vp;

    old_head = head;
    if ((head = head->next) == NULL)
        tail = NULL;

    if (old_head != &signal_request)
        free (old_head);
    else
        old_head->next = NULL;

    return 1;
}
